@@ -34,11 +34,13 @@ use PPIx::QuoteLike::Utils qw{
     statement
     visual_column_number
     __instance
+    __match_enclosed
+    __matching_delimiter
 };
 use Scalar::Util ();
 use Text::Tabs ();
 
-our $VERSION = '0.010';
+our $VERSION = '0.016';
 
 use constant CLASS_CONTROL       => 'PPIx::QuoteLike::Token::Control';
 use constant CLASS_DELIMITER     => 'PPIx::QuoteLike::Token::Delimiter';
@@ -54,21 +56,24 @@ use constant ILLEGAL_FIRST	=>
     'Tokenizer found illegal first characters';
 use constant MISMATCHED_DELIM	=>
     'Tokenizer found mismatched delimiters';
-
-$PPIx::QuoteLike::DEFAULT_POSTDEREF = 1;
+use constant NO_INDENTATION	=>
+    'No indentation string found';
 
 {
-    my $match_sq = _match_enclosed( qw< ' > );
-    my $match_dq = _match_enclosed( qw< " > );
-    my $match_bt = _match_enclosed( qw< ` > );
+    my $match_sq = __match_enclosed( qw< ' > );
+    my $match_dq = __match_enclosed( qw< " > );
+    my $match_bt = __match_enclosed( qw< ` > );
 
     sub new {	## no critic (RequireArgUnpacking)
 	my ( $class, $source, %arg ) = @_;
 
 	my @children;
 
-	defined $arg{postderef}
-	    or $arg{postderef} = $PPIx::QuoteLike::DEFAULT_POSTDEREF;
+	if ( defined $arg{postderef} ) {
+	    $class->_deprecation_notice( attribute => 'postderef' );
+	} else {
+	    $arg{postderef} = 1;
+	}
 
 	if ( $arg{location} ) {
 	    ARRAY_REF eq ref $arg{location}
@@ -99,7 +104,7 @@ $PPIx::QuoteLike::DEFAULT_POSTDEREF = 1;
 	defined( my $string = $self->_stringify_source( $source ) )
 	    or return;
 
-	my ( $type, $gap, $content, $end_delim, $start_delim );
+	my ( $type, $gap, $gap2, $content, $end_delim, $indented, $start_delim );
 
 	$arg{trace}
 	    and warn "Initial match of $string\n";
@@ -116,7 +121,7 @@ $PPIx::QuoteLike::DEFAULT_POSTDEREF = 1;
 	    $self->{interpolates} = 'qq' eq $type ||
 		'qx' eq $type && q<'> ne $start_delim;
 	    $content = substr $string, ( pos $string || 0 );
-	    $end_delim = _matching_delimiter( $start_delim );
+	    $end_delim = __matching_delimiter( $start_delim );
 	    if ( $end_delim eq substr $content, -1 ) {
 		chop $content;
 	    } else {
@@ -127,15 +132,28 @@ $PPIx::QuoteLike::DEFAULT_POSTDEREF = 1;
 	# Note that the regexp used here is slightly wrong in that white
 	# space between the '<<' and the termination string is not
 	# allowed if the termination string is not quoted in some way.
-	} elsif ( $string =~ m/ \A \s* ( << ) ( \s* )
-	    ( \w+ | $match_sq | $match_dq | $match_bt ) \n /smxgc ) {
-	    ( $type, $gap, $start_delim ) = ( $1, $2, $3 );
+	} elsif ( $string =~ m/ \A \s* ( << ) ( \s* ) ( ~? ) ( \s* )
+	    ( [\\]? \w+ | $match_sq | $match_dq | $match_bt ) \n /smxgc ) {
+	    ( $type, $gap, $indented, $gap2, $start_delim ) = (
+		$1, $2, $3, $4, $5 );
 	    $arg{trace}
-		and warn "Initial match '$type$start_delim$gap'\n";
-	    $self->{interpolates} = $start_delim !~ m/ \A ' /smx;
+		and warn "Initial match '$type$start_delim$gap$indented'\n";
+	    $self->{interpolates} = $start_delim !~ m/ \A [\\'] /smx;
 	    $content = substr $string, ( pos $string || 0 );
 	    $end_delim = _unquote( $start_delim );
-	    if ( $content =~ s/ ^ \Q$end_delim\E \n? \z //smx ) {
+	    # NOTE that the indentation is specifically space or tab
+	    # only.
+	    if ( $content =~ s/ ^ ( [ \t]* ) \Q$end_delim\E \n? \z //smx ) {
+		# NOTE PPI::Token::HereDoc does not preserve the
+		# indentation of an indented here document, so the
+		# indentation will appear to be '' if we came from PPI.
+		if ( $indented ) {
+		    # Version per perldelta.pod for that release.
+		    $self->{perl_version_introduced} = '5.025007';
+		    $self->{indentation} = "$1";
+		    $self->{_indentation_re} = qr/
+			^ \Q$self->{indentation}\E /smx;
+		}
 	    } else {
 		$end_delim = '';
 	    }
@@ -157,7 +175,7 @@ $PPIx::QuoteLike::DEFAULT_POSTDEREF = 1;
 		and warn "Initial match '$type$start_delim'\n";
 	    $self->{interpolates} = q<'> ne $start_delim;
 	    $content = substr $string, ( pos $string || 0 );
-	    $end_delim = _matching_delimiter( $start_delim );
+	    $end_delim = __matching_delimiter( $start_delim );
 	    if ( $end_delim eq substr $content, -1 ) {
 		chop $content;
 	    } else {
@@ -179,6 +197,12 @@ $PPIx::QuoteLike::DEFAULT_POSTDEREF = 1;
 	    length $gap ?
 		$self->_make_token( CLASS_WHITESPACE, $gap ) :
 		(),
+	    length $indented ?
+	        $self->_make_token( CLASS_STRUCTURE, $indented ) :
+		(),
+	    length $gap2 ?
+		$self->_make_token( CLASS_WHITESPACE, $gap2 ) :
+		(),
 	];
 	$self->{start} ||= [
 	    $self->_make_token( CLASS_DELIMITER, $start_delim ),
@@ -195,13 +219,15 @@ $PPIx::QuoteLike::DEFAULT_POSTDEREF = 1;
 
 		if ( $content =~ m/ \G ( \\ [ULulQEF] ) /smxgc ) {
 		    push @children, [ CLASS_CONTROL, "$1" ];
-		} elsif ( $content =~ m/ \G ( \\ N [{] ( [^}]+ ) [}] ) /smxgc ) {
+		} elsif (
+		    $content =~ m/ \G ( \\ N [{] ( [^}]+ ) [}] ) /smxgc
+		) {
 		    # Handle \N{...} separately because it can not
 		    # contain an interpolation even inside of an
 		    # otherwise-interpolating string. That is to say,
 		    # "\N{$foo}" is simply invalid, and does not even
 		    # try to interpolate $foo.  {
-		    # TODO use $re = _match_enclosed( '{' ); # }
+		    # TODO use $re = __match_enclosed( '{' ); # }
 		    my ( $seq, $name ) = ( $1, $2 );
 		    # TODO The Regexp is certainly too permissive. For
 		    # the moment all I am doing is disallowing
@@ -210,22 +236,23 @@ $PPIx::QuoteLike::DEFAULT_POSTDEREF = 1;
 			[ CLASS_UNKNOWN, $seq,
 			    error => "Unknown charname '$name'" ] :
 			[ CLASS_STRING, $seq ];
+		# NOTE in the following that I do not read perldata as
+		# saying there can be space between the sigil and the
+		# variable name, but Perl itself seems to accept it as
+		# of 5.30.1.
 		} elsif ( $content =~ m/ \G ( [\$\@] \#? \$* ) /smxgc ) {
 		    push @children, $self->_interpolation( "$1", $content );
 		} elsif ( $content =~ m/ \G ( \\ . | [^\\\$\@]+ ) /smxgc ) {
-		    push @children, [ CLASS_STRING, "$1" ];
+		    push @children, $self->_remove_here_doc_indentation(
+			"$1",
+			sibling	=> \@children,
+		    );
 		} else {
 		    last;
 		}
-	    } continue {
-		# We might have consecutive strings for various reasons.
-		# Merge these.
-		if ( CLASS_STRING eq $children[-1][0] &&
-		    CLASS_STRING eq $children[-2][0] ) {
-		    my $merge = pop @children;
-		    $children[-1][1] .= $merge->[1];
-		}
 	    }
+
+	    @children = _merge_strings( @children );
 	    shift @children;	# remove the priming
 
 	    # Make the tokens, at long last.
@@ -236,11 +263,18 @@ $PPIx::QuoteLike::DEFAULT_POSTDEREF = 1;
 	} else {
 
 	    length $content
-		and push @children, $self->_make_token(
-		    CLASS_STRING, $content );
+		and push @children, map { $self->_make_token( @{ $_ } ) }
+		    _merge_strings(
+			$self->_remove_here_doc_indentation( $content )
+			);
 
 	}
 
+	# Add the indentation before the end marker, if needed
+	$self->{indentation}
+	    and push @children, $self->_make_token(
+	    CLASS_WHITESPACE, $self->{indentation} );
+
 	if ( $self->{finish} ) {
 	    # If we already have something here it is data, not objects.
 	    foreach ( @{ $self->{finish} } ) {
@@ -281,6 +315,48 @@ sub delimiters {
 	qw{ start finish };
 }
 
+#	$self->_deprecation_notice( $type, $name );
+#
+#	This method centralizes deprecation. Type is 'attribute' or
+#	'method'. Deprecation is driven of the %deprecate hash. Values
+#	are:
+#	    false - no warning
+#	    1 - warn on first use
+#	    2 - warn on each use
+#	    3 - die on each use.
+#
+#	$self->_deprecation_in_progress( $type, $name )
+#
+#	This method returns true if the deprecation is in progress. In
+#	fact it returns the deprecation level.
+
+{
+
+    my %deprecate = (
+	attribute => {
+	    postderef	=> 1,
+	},
+    );
+
+    sub _deprecation_notice {
+	my ( undef, $type, $name, $repl ) = @_;		# Invocant unused
+	$deprecate{$type} or return;
+	$deprecate{$type}{$name} or return;
+	my $msg = sprintf 'The %s %s is %s', $name, $type,
+	    $deprecate{$type}{$name} > 2 ? 'removed' : 'deprecated';
+	defined $repl
+	    and $msg .= "; use $repl instead";
+	$deprecate{$type}{$name} >= 3
+	    and croak $msg;
+	warnings::enabled( 'deprecated' )
+	    and carp $msg;
+	$deprecate{$type}{$name} == 1
+	    and $deprecate{$type}{$name} = 0;
+	return;
+    }
+
+}
+
 sub _get_value_scalar {
     my ( $self, $method ) = @_;
     defined( my $val = $self->$method() )
@@ -337,6 +413,11 @@ sub handles {
     return $self->_stringify_source( $string, test => 1 );
 }
 
+sub indentation {
+    my ( $self ) = @_;
+    return $self->{indentation};
+}
+
 sub interpolates {
     my ( $self ) = @_;
     return $self->{interpolates};
@@ -424,6 +505,9 @@ sub perl_version_removed {
 
 sub postderef {
     my ( $self ) = @_;
+    # TODO postderef - eventually this goes away.
+    __PACKAGE__ eq caller
+	or $self->_deprecation_notice( attribute => 'postderef' );
     return $self->{postderef};
 }
 
@@ -573,7 +657,6 @@ sub __decode {
 
 }
 
-
 # This subroutine was created in an attempt to simplify control flow.
 # Argument 2 (from 0) is not unpacked because the caller needs to see
 # the side effects of matches made on it.
@@ -604,7 +687,7 @@ sub __decode {
 
 	if ( $_[2] =~ m/ \G (?= \{ ) /smxgc ) {
 	    # variable name enclosed in {}
-	    my $delim_re = _match_enclosed( qw< { > );
+	    my $delim_re = __match_enclosed( qw< { > );
 	    $_[2] =~ m/ \G ( $delim_re ) /smxgc
 		and return [ CLASS_INTERPOLATION, "$sigil$1" ];
 	    $_[2] =~ m/ \G ( .* ) /smxgc
@@ -619,7 +702,7 @@ sub __decode {
 	    my $interp = "$sigil$1";
 	    while ( $_[2] =~ m/ \G  ( (?: -> )? ) (?= ( [[{] ) ) /smxgc ) { # }]
 		my $lead_in = $1;
-		my $delim_re = _match_enclosed( $2 );
+		my $delim_re = __match_enclosed( $2 );
 		if ( $_[2] =~ m/ \G ( $delim_re ) /smxgc ) {
 		    $interp .= "$lead_in$1";
 		} else {
@@ -670,81 +753,6 @@ sub _link_elems {
     return $self;
 }
 
-{
-    our %REGEXP_CACHE;
-
-    my %matching_bracket = qw/ ( ) [ ] { } < > /;
-
-    sub _match_enclosed {
-	my ( $left ) = @_;
-	my $ql = quotemeta $left;
-	$REGEXP_CACHE{$ql}
-	    and return $REGEXP_CACHE{$ql};
-	if ( my $right = $matching_bracket{$left} ) {
-
-=begin comment
-
-	    return ( $REGEXP_CACHE{$left} =
-		qr/ (
-		    \Q$left\E
-		    (?:
-			(?> [^\\\Q$left$right\E]+ ) |
-			(?> \$ [\Q$left$right\E] ) |
-			(?> \\ . ) |
-			(?-1)
-		    )*
-		    \Q$right\E
-		) /smx
-	    );
-
-=end comment
-
-=cut
-
-	    # Based on Regexp::Common $RE{balanced} 2.113 (because I
-	    # can't use (?-1)
-
-	    my $ql = quotemeta $left;
-	    my $qr = quotemeta $right;
-	    my $pkg = __PACKAGE__;
-	    my $r  = "(??{ \$${pkg}::REGEXP_CACHE{'$ql'} })";
-
-	    my @parts = (
-		"(?>[^\\\\$ql$qr]+)",
-		"(?>\\\$[$ql$qr])",
-		'(?>\\\\.)',
-		$r,
-	    );
-
-	    {
-		use re qw{ eval };
-		local $" = '|';
-		$REGEXP_CACHE{$ql} = qr/($ql(?:@parts)*$qr)/;
-	    }
-
-	    return $REGEXP_CACHE{$ql};
-
-	} else {
-
-	    # Based on Regexp::Common $RE{delimited}{-delim=>'`'}
-	    return ( $REGEXP_CACHE{$ql} ||=
-		qr< (?:
-		    (?: \Q$left\E )
-		    (?: [^\\\Q$left\E]* (?: \\ . [^\\\Q$left\E]* )* )
-		    (?: \Q$left\E )
-		) >smx
-	    );
-	}
-    }
-
-    sub _matching_delimiter {
-	my ( $left ) = @_;
-	my $right = $matching_bracket{$left}
-	    or return $left;
-	return $right;
-    }
-}
-
 {
     my %allow_subscr	= map { $_ => 1 } qw{ % @ };
 
@@ -766,7 +774,7 @@ sub _link_elems {
 	    $allow_subscr{$sigil} &&
 	    $_[0] =~ m/ \G (?= ( [[{] ) ) /smxgc	# }]
 	) {
-	    my $re = _match_enclosed( "$1" );
+	    my $re = __match_enclosed( "$1" );
 	    $_[0] =~ m/ \G $re /smxgc
 		and return "$match$1";
 	}
@@ -776,6 +784,89 @@ sub _link_elems {
     }
 }
 
+# For various reasons we may get consecutive literals -- typically
+# strings. We want to merge these. The arguments are array refs, with
+# the class name of the token in [0] and the content in [1]. I know of
+# no way we can generate consecutive white space tokens, but if I did I
+# would want them merged.
+#
+# NOTE that merger loses all attributes of the second token, so we MUST
+# NOT merge CLASS_UNKNOWN tokens, or any class that might have
+# attributes other than content.
+{
+    my %can_merge = map { $_ => 1 } CLASS_STRING, CLASS_WHITESPACE;
+
+    sub _merge_strings {
+	my @arg = @_;
+	my @rslt;
+	foreach my $elem ( @arg ) {
+	    if ( @rslt && $can_merge{$elem->[0]}
+		&& $elem->[0] eq $rslt[-1][0]
+	    ) {
+		$rslt[-1][1] .= $elem->[1];
+	    } else {
+		push @rslt, $elem;
+	    }
+	}
+	return @rslt;
+    }
+}
+
+# If we're processing an indented here document, strings must be split
+# on new lines and un-indented. We return array refs rather than
+# objects because we may be called before we're ready to build the
+# objects.
+sub _remove_here_doc_indentation {
+    my ( $self, $string, %arg ) = @_;
+
+    # NOTE that we rely on the fact that both undef (not indented) and
+    # '' (indented by zero characters) evaluate false.
+    $self->{indentation}
+	or return [ CLASS_STRING, $string ];
+
+    my $ignore_first;
+    if ( $arg{sibling} ) {
+	# Because the calling code primes the pump, @sibling will never
+	# be empty, even when processing the first token. So:
+	# * The pump-priming specifies class '', so if that is what we
+	#   see we must process the first line; otherwise
+	# * If the previous token is a string ending in "\n", we must
+	#   process the first line.
+	$ignore_first = '' ne $arg{sibling}[-1][0] && (
+	    CLASS_STRING ne $arg{sibling}[-1][0] ||
+	    $arg{sibling}[-1][1] !~ m/ \n \z /smx );
+    } else {
+	# Without @sibling, we unconditionally process the first line.
+	$ignore_first = 0;
+    }
+
+    my @rslt;
+
+    foreach ( split qr/ (?<= \n ) /smx, $string ) {
+	if ( $ignore_first ) {
+	    push @rslt, [ CLASS_STRING, "$_" ];
+	    $ignore_first = 0;
+	} else {
+	    if ( "\n" eq $_ ) {
+		push @rslt,
+		    [ CLASS_STRING, "$_" ],
+		    ;
+	    } elsif ( s/ ( $self->{_indentation_re} ) //smx ) {
+		push @rslt,
+		    [ CLASS_WHITESPACE, "$1" ],
+		    [ CLASS_STRING, "$_" ],
+		    ;
+	    } else {
+		push @rslt,
+		    [ CLASS_UNKNOWN, "$_", error => NO_INDENTATION ],
+		    ;
+	    }
+	}
+    }
+
+    return @rslt;
+}
+
 sub _stringify_source {
     my ( $self, $string, %opt ) = @_;
 
@@ -859,6 +950,57 @@ reasonably like string literals. Its real reason for being is to find
 interpolated variables for L<Perl::Critic|Perl::Critic> policies and
 similar code.
 
+The parse is fairly straightforward, and a little poking around with
+F<eg/pqldump> should show how it normally goes.
+
+But there is at least one quote-like thing that probably needs some
+explanation.
+
+=head2 Indented Here Documents
+
+These were introduced in Perl 5.25.7 (November 2016) but not recognized
+by this module until its version 0.015 (February 2021). The indentation
+is parsed as
+L<PPIx::QuoteLike::Token::Whitespace|PPIx::Regexp::Token::Whitespace>
+objects, provided it is at least one character wide, otherwise it is not
+represented in the parse. That is to say,
+
+ <<~EOD
+     How doth the little crocodile
+     Improve his shining tail
+     EOD
+
+will have the three indentations represented by whitespace objects and
+each line of the literal represented by its own string object, but
+
+ <<~EOD
+ How doth the little crocodile
+ Improve his shining tail
+ EOD
+
+will parse the same as the non-indented version, except for the addition
+of the token representing the C<'~'>.
+
+L<PPI|PPI> is ahead of this module, and recognized indented here
+documents as of its version 1.246 (May 2019). Unfortunately, as of
+version 1.270 the indent gets lost in the parse, so a C<PPIx::QuoteLike>
+object initialized from such a
+L<PPI::Token::HereDoc|PPI::Token::HereDoc> will be seen as having an
+indentation of C<''> regardless of the actual indentation in the source.
+I believe this restriction will go away when
+L<https://github.com/adamkennedy/PPI/issues/251> is resolved.
+
+=head1 DEPRECATION NOTICE
+
+The L<postderef|/postderef> argument to L<new()|/new> is being put
+through a deprecation cycle and retracted. After the retraction, postfix
+dereferences will always be recognized. This is the default behaviour
+now.
+
+Starting with version 0.012_01, the first use of this argument will
+warn. With the first release after April 8 2020, all uses will warn.
+After a further six months, all uses will become fatal.
+
 =head1 INHERITANCE
 
 C<PPIx::QuoteLike> is not descended from any other class.
@@ -882,7 +1024,8 @@ quote-like literal of some sort, provided it begins like one. Otherwise
 this method will return nothing. The scalar representation of a here
 document is a multi-line string whose first line consists of the leading
 C< << > and the start delimiter, and whose subsequent lines consist of
-the content of the here document and the end delimiter.
+the content of the here document and the end delimiter. Indented here
+documents were not supported by this class until version C<0.015>.
 
 C<PPI> classes that can be handled are
 L<PPI::Token::Quote|PPI::Token::Quote>,
@@ -923,13 +1066,14 @@ L<PPI::Element|PPI::Element>, otherwise no locations will be available.
 
 =item postderef
 
+B<THIS ARGUMENT IS DEPRECATED>.
+See L<DEPRECATION NOTICE|/DEPRECATION NOTICE> above for the details.
+
 This Boolean argument determines whether postfix dereferencing is
 recognized in interpolation. If unspecified, or specified as C<undef>,
-it defaults to the value of C<$PPIx::QuoteLike::DEFAULT_POSTDEREF>. This
-variable is not exported, and is true by default. If you change the
-value, the change should be properly localized:
-
- local $PPIx::QuoteLike::DEFAULT_POSTDEREF = 0;
+it defaults to true. In version 0.012 it defaulted to the value of
+C<$PPIx::QuoteLike::DEFAULT_POSTDEREF>. This variable was not exported,
+and was true by default.
 
 =item trace
 
@@ -1042,6 +1186,15 @@ This convenience static method returns a true value if this package can
 be expected to handle the content of C<$string> (be it scalar or
 object), and a false value otherwise.
 
+=head2 indentation
+
+This method returns the indentation string if the object represents an
+indented here document, or C<undef> if it represents anything else,
+including an unindented here document.
+
+B<Note> that if indented syntax is used but the here document is not in
+fact indented, this will return C<''>, which evaluates to false.
+
 =head2 interpolates
 
  say $str->interpolates() ?
@@ -1180,6 +1333,13 @@ or element zero if no argument is specified.
 
  say "Interpolates $_" for $str->variables();
 
+B<NOTE> that this method is discouraged, and may well be deprecated and
+removed. My problem with it is that it returns variable names rather
+than L<PPI::Element|PPI::Element> objects, leaving you no idea how the
+variables are used. It was originally written for the benefit of
+L<Perl::Critic::Policy::Variables::ProhibitUnusedVarsStricter|Perl::Critic::Policy::Variables::ProhibitUnusedVarsStricter>,
+but has proven inadequate to that policy's needs.
+
 This convenience method returns all interpolated variables. Each is
 returned only once, and they are returned in no particular order. If the
 object does not represent a string that interpolates, nothing is
@@ -1221,6 +1381,12 @@ things. I do not have any examples of where the parse of a quote-like
 thing would change based on what is interpolated, but neither can I rule
 it out. I<Caveat user>.
 
+=head2 PPI Restrictions
+
+As of version 0.015 of this module, the only known instance of this is
+the handling of indented here documents, as discussed above under
+L<Indented Here Documents|/Indented Here Documents>.
+
 =head2 Non-Standard Syntax
 
 There are modules out there that alter the syntax of Perl. If the syntax
@@ -1242,7 +1408,9 @@ L<Perl6::Pugs|Perl6::Pugs>. Enough said.
 =head1 SUPPORT
 
 Support is by the author. Please file bug reports at
-L<https://rt.cpan.org>, or in electronic mail to the author.
+L<https://rt.cpan.org/Public/Dist/Display.html?Name=PPIx-QuoteLike>,
+L<https://github.com/trwyant/perl-PPIx-QuoteLike/issues>, or in
+electronic mail to the author.
 
 =head1 AUTHOR
 
@@ -1250,7 +1418,7 @@ Thomas R. Wyant, III F<wyant at cpan dot org>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2016-2020 by Thomas R. Wyant, III
+Copyright (C) 2016-2021 by Thomas R. Wyant, III
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl 5.10.0. For more details, see the full text
